@@ -8,6 +8,7 @@
 #define HAS_LIVES_MAINWINDOW_H
 
 #include <pthread.h>
+#include <inttypes.h>
 
 #include "effects-data.h"
 
@@ -48,17 +49,17 @@
 /////// GUI related constants /////////////////////////////////////////////////////////
 
 // parameters for resizing the image frames, and for capture
-#define V_RESIZE_ADJUST ((W_PACKING_WIDTH+2)*3)
-#define H_RESIZE_ADJUST ((W_PACKING_HEIGHT+2)*2)
+#define V_RESIZE_ADJUST ((W_PACKING_WIDTH + 2) * 3)
+#define H_RESIZE_ADJUST ((W_PACKING_HEIGHT + 2) * 2)
 
 #if GTK_CHECK_VERSION(3, 0, 0)
-#define CE_FRAME_HSPACE ((int)(320.*widget_opts.scale))
+#define CE_FRAME_HSPACE ((int)(320. * widget_opts.scale))
 #else
-#define CE_FRAME_HSPACE ((int)(420.*widget_opts.scale))
+#define CE_FRAME_HSPACE ((int)(420. * widget_opts.scale))
 #endif
 
-/// char width of start / end spinbuttons
-#define SPBWIDTHCHARS 12
+/// char width of combo entries (default)
+#define COMBOWIDTHCHARS 12
 
 /// char width of framecounter
 #define FCWIDTHCHARS 18
@@ -70,59 +71,55 @@
 #define MIN_SEPWIN_HEIGHT 36
 
 /// sepwin/screen size safety margins in pixels
-#define SCR_WIDTH_SAFETY ((int)(100.*widget_opts.scale))
-#define SCR_HEIGHT_SAFETY ((int)(200.*widget_opts.scale))
+#define SCR_WIDTH_SAFETY ((int)(100. * widget_opts.scale))
+#define SCR_HEIGHT_SAFETY ((int)(200. * widget_opts.scale))
 
 /// default size for generators
-#define DEF_GEN_WIDTH 640
-#define DEF_GEN_HEIGHT 480
+#define DEF_GEN_WIDTH 1024
+#define DEF_GEN_HEIGHT 720
 
 /// height of preview widgets in sepwin
-#define PREVIEW_BOX_HT ((int)(100.*widget_opts.scale))
-
-/// height of msg area
-#define MSG_AREA_HEIGHT ((int)(50.*widget_opts.scale))
-
-/// clip editor hrule height
-#define CE_HRULE_HEIGHT ((int)(20.*widget_opts.scale))
-
-/// clip edit vid/aud bar height
-#define CE_VIDBAR_HEIGHT ((int)(10.*widget_opts.scale))
+#define PREVIEW_BOX_HT ((int)(100. * widget_opts.scale))
 
 /// (unexpanded) height of rows in treeviews
-#define TREE_ROW_HEIGHT ((int)(60.*widget_opts.scale))
+#define TREE_ROW_HEIGHT ((int)(60. * widget_opts.scale))
 
 // a few GUI specific settings
 #define GUI_SCREEN_WIDTH (mainw->mgeom[widget_opts.monitor].width)
 #define GUI_SCREEN_HEIGHT (mainw->mgeom[widget_opts.monitor].height)
+#define GUI_SCREEN_X (mainw->mgeom[widget_opts.monitor].x)
+#define GUI_SCREEN_Y (mainw->mgeom[widget_opts.monitor].y)
 
-#define SCREEN_SCALE_DEF_WIDTH 1200
+#define SCREEN_SCALE_DEF_WIDTH 1600
 
-#define DEFAULT_FRAME_HSIZE ((int)(640.*widget_opts.scale))
-#define DEFAULT_FRAME_VSIZE ((int)(400.*widget_opts.scale))
+#define DEFAULT_FRAME_HSIZE_UNSCALED 640
+#define DEFAULT_FRAME_VSIZE_UNSCALED 480
 
-#define FRAMEBLANK_MIN_WIDTH ((int)(240.*widget_opts.scale))
-#define FRAMEBLANK_MAX_WIDTH ((int)(600.*widget_opts.scale))
+#define DEFAULT_FRAME_HSIZE ((int)((double)DEFAULT_FRAME_HSIZE_UNSCALED * widget_opts.scale))
+#define DEFAULT_FRAME_VSIZE ((int)((double)DEFAULT_FRAME_VSIZE_UNSCALED * widget_opts.scale))
 
-#define FRAMEBLANK_MIN_HEIGHT ((int)(180.*widget_opts.scale))
-#define FRAMEBLANK_MAX_HEIGHT ((int)(400.*widget_opts.scale))
+#define FRAMEBLANK_MIN_WIDTH ((int)(240. * widget_opts.scale))
+#define FRAMEBLANK_MAX_WIDTH ((int)(600. * widget_opts.scale))
 
-#define IMSEP_MAX_HEIGHT ((int)(86.*widget_opts.scale))
+#define FRAMEBLANK_MIN_HEIGHT ((int)(180. * widget_opts.scale))
+#define FRAMEBLANK_MAX_HEIGHT ((int)(400. * widget_opts.scale))
+
+#define IMSEP_MAX_HEIGHT ((int)(86. * widget_opts.scale))
 #define IMSEP_MAX_WIDTH (GUI_SCREEN_WIDTH - 20)
 
-#define MAIN_SPIN_SPACER ((int)52.*widget_opts.scale) ///< pixel spacing for start/end spins for clip and multitrack editors
+#define MAIN_SPIN_SPACER ((int)52. * widget_opts.scale) ///< pixel spacing for start/end spins for clip and multitrack editors
 
-#define ENC_DETAILS_WIN_H ((int)(640.*widget_opts.scale)) ///< horizontal size in pixels of the encoder output window
-#define ENC_DETAILS_WIN_V ((int)(240.*widget_opts.scale)) ///< vertical size in pixels of the encoder output window
+#define ENC_DETAILS_WIN_H ((int)(640. * widget_opts.scale)) ///< horizontal size in pixels of the encoder output window
+#define ENC_DETAILS_WIN_V ((int)(240. * widget_opts.scale)) ///< vertical size in pixels of the encoder output window
 
-#define MIN_MSG_WIDTH_CHARS ((int)(40.*widget_opts.scale)) ///< min width of text on warning/error labels
-#define MAX_MSG_WIDTH_CHARS ((int)(100.*widget_opts.scale)) ///< max width of text on warning/error labels
+#define MIN_MSG_WIDTH_CHARS ((int)(40. * widget_opts.scale)) ///< min width of text on warning/error labels
+#define MAX_MSG_WIDTH_CHARS ((int)(100. * widget_opts.scale)) ///< max width of text on warning/error labels
 
 /// size of the fx dialog windows scrollwindow
-#define RFX_WINSIZE_H ((int)(GUI_SCREEN_WIDTH >= 1024 ? (820.*widget_opts.scale) : 640))
-#define RFX_WINSIZE_V ((int)(480.*widget_opts.scale))
+#define RFX_WINSIZE_H ((int)(GUI_SCREEN_WIDTH >= SCREEN_SCALE_DEF_WIDTH ? 210. + 210. * widget_opts.scale : 640))
+#define RFX_WINSIZE_V ((int)(480. * widget_opts.scale))
 
-#define DEF_BUTTON_WIDTH ((int)(80.*widget_opts.scale))
+#define DEF_BUTTON_WIDTH ((int)(80. * widget_opts.scale))
 
 #define DEF_DIALOG_WIDTH RFX_WINSIZE_H
 #define DEF_DIALOG_HEIGHT RFX_WINSIZE_V
@@ -183,7 +180,8 @@ typedef enum {
   LIVES_DIALOG_WARN_WITH_CANCEL,
   LIVES_DIALOG_YESNO,
   LIVES_DIALOG_QUESTION,
-  LIVES_DIALOG_ABORT_CANCEL_RETRY
+  LIVES_DIALOG_ABORT_CANCEL_RETRY,
+  LIVES_DIALOG_CANCEL_RETRY_BROWSE
 } lives_dialog_t;
 
 /// various return conditions from rendering (multitrack or after recording)
@@ -255,6 +253,8 @@ typedef struct {
 #define STYLE_3 (1<<2) ///< style is lightish - allow themeing of widgets with dark text, otherwise use menu bg
 #define STYLE_4 (1<<3) ///< separator col. in mt
 
+#define STYLE_LIGHT STYLE_3
+
   LiVESWidgetColor white;
   LiVESWidgetColor black;
   LiVESWidgetColor pink;
@@ -321,6 +321,8 @@ enum {
   NUM_LIVES_STRING_CONSTANTS
 };
 
+#define BIN_YOUTUBE_DL "youtube-dl"
+
 // image types (string)
 #define LIVES_IMAGE_TYPE_JPEG "jpeg"
 #define LIVES_IMAGE_TYPE_PNG "png"
@@ -333,10 +335,12 @@ enum {
 #define LIVES_FILE_EXT_JPG "jpg"
 #define LIVES_FILE_EXT_MGK "mgk"
 #define LIVES_FILE_EXT_PRE "pre"
+#define LIVES_FILE_EXT_NEW "new"
 #define LIVES_FILE_EXT_SCRAP "scrap"
 #define LIVES_FILE_EXT_TEXT "txt"
 #define LIVES_FILE_EXT_BAK "bak"
 #define LIVES_FILE_EXT_WEBM "webm"
+#define LIVES_FILE_EXT_MP4 "mp4"
 
 #define LIVES_FILE_EXT_BACKUP "lv1"
 #define LIVES_FILE_EXT_PROJECT "lv2"
@@ -418,8 +422,10 @@ typedef struct {
 #define RFX_TOOL_MENU_POSN 2
 
 /// mainw->
+#define MAINW_MSG_SIZE 8192
+
 typedef struct {
-  char msg[512];
+  char msg[8192];
 
   // files
   int current_file;
@@ -473,6 +479,7 @@ typedef struct {
   boolean ccpd_with_sound;
   boolean selwidth_locked;
   boolean is_ready;
+  boolean configured;
   boolean fatal; ///< got fatal signal
   boolean opening_loc;  ///< opening location (streaming)
   boolean dvgrab_preview;
@@ -557,7 +564,7 @@ typedef struct {
   boolean last_transition_align_start;
   boolean last_transition_ins_frames;
 
-  uint64_t rte; ///< current max for VJ mode == 64 effects on fg clip
+  volatile uint64_t rte; ///< current max for VJ mode == 64 effects on fg clip
 
   uint32_t last_grabbable_effect;
   int rte_keys; ///< which effect is bound to keyboard
@@ -579,7 +586,6 @@ typedef struct {
 
   lives_toy_t toy_type;
   lives_pgid_t alives_pgid; // 0, or procid for autolives
-  boolean autolives_reset_fx;
 
   boolean toy_go_wild;
 
@@ -616,7 +622,7 @@ typedef struct {
   int old_scr_height;
 
   char first_info_file[PATH_MAX];
-  boolean leave_files;
+  boolean leave_files;  ///< TRUE to leave clip files on disk even when closing (default FALSE)
   boolean was_set;
 
   /// extra parameters for opening special files
@@ -636,12 +642,12 @@ typedef struct {
 
   // for the internal player
   double period; ///< == 1./cfile->pb_fps (unless cfile->pb_fps is 0.)
-  uint64_t startticks; ///< effective ticks when last frame was (should have been) displayed
+  volatile uint64_t startticks; ///< effective ticks when last frame was (should have been) displayed
   uint64_t timeout_ticks; ///< incremented if effect/rendering is paused/previewed
   uint64_t origsecs; ///< playback start seconds - subtracted from all other ticks to keep numbers smaller
   uint64_t origusecs; ///< usecs at start of playback - ditto
-  uint64_t offsetticks; ///< offset for external transport
-  uint64_t currticks; ///< current playback ticks (relative)
+  uint64_t offsetticks; ///< offset for multitrack playback start
+  volatile uint64_t currticks; ///< current playback ticks (relative)
   uint64_t deltaticks; ///< deltaticks for scratching
   uint64_t firstticks; ///< ticks when audio started playing (for non-realtime audio plugins)
   uint64_t stream_ticks;  ///< ticks since first frame sent to playback plugin
@@ -740,6 +746,7 @@ typedef struct {
   ulong mute_audio_func;
   ulong fullscreen_cb_func;
   ulong sepwin_cb_func;
+  ulong sw_func; ///< scrolledwindow expose func
 
   // for jack transport
   boolean jack_can_stop;
@@ -767,6 +774,7 @@ typedef struct {
   LiVESWidget *freventbox1;
   LiVESWidget *playframe;
   LiVESWidget *pl_eventbox;
+  LiVESWidget *pf_grid;
   LiVESPixbuf *imframe;
   LiVESPixbuf *camframe;
   LiVESPixbuf *imsep;
@@ -793,10 +801,7 @@ typedef struct {
   LiVESWidget *tvdev;
   LiVESWidget *recent_menu;
   LiVESWidget *recent_submenu;
-  LiVESWidget *recent1;
-  LiVESWidget *recent2;
-  LiVESWidget *recent3;
-  LiVESWidget *recent4;
+  LiVESWidget *recent[N_RECENT_FILES];
   LiVESWidget *save_as;
 #ifdef LIBAV_TRANSCODE
   LiVESWidget *transcode;
@@ -882,7 +887,6 @@ typedef struct {
   LiVESWidget *resample_video;
   LiVESWidget *preferences;
   LiVESWidget *rename;
-  LiVESWidget *toys;
   LiVESWidget *toy_none;
   LiVESWidget *toy_random_frames;
   LiVESWidget *toy_tv;
@@ -896,7 +900,6 @@ typedef struct {
   LiVESAccelGroup *accel_group;
   LiVESWidget *sep_image;
   LiVESWidget *hruler;
-  LiVESWidget *vj_menu;
   LiVESWidget *vj_save_set;
   LiVESWidget *vj_load_set;
   LiVESWidget *vj_show_keys;
@@ -904,6 +907,7 @@ typedef struct {
   LiVESWidget *rte_defs;
   LiVESWidget *save_rte_defs;
   LiVESWidget *vj_reset;
+  LiVESWidget *vj_realize;
   LiVESWidget *mt_menu;
   LiVESWidget *troubleshoot;
   LiVESWidget *export_custom_rfx;
@@ -926,11 +930,16 @@ typedef struct {
   LiVESWidget *framedraw_preview; ///< the 'redraw' button
   LiVESWidget *framedraw_spinbutton; ///< the frame number button
   LiVESWidget *framedraw_scale; ///< the slider
+  LiVESWidget *framedraw_maskbox; ///< box for opacity controls
+  LiVESWidget *framedraw_opscale; ///< opacity
+  LiVESWidget *framedraw_cbutton; ///< colour for mask
   LiVESWidget *fd_frame; ///< surrounding frame widget
 
   weed_plant_t *fd_layer_orig; ///< original layer uneffected
   weed_plant_t *fd_layer; ///< framedraw preview layer
 
+  LiVESWidget *hbox3;  ///< hbox with start / end spins and selection label (C.E.)
+
   // bars here -> actually text above bars
   LiVESWidget *vidbar;
   LiVESWidget *laudbar;
@@ -941,6 +950,7 @@ typedef struct {
 
   LiVESWidget *sa_button;
   LiVESWidget *sa_hbox;
+  LiVESWidget *sa_toolbar;
 
   LiVESWidget *arrow1;
   LiVESWidget *arrow2;
@@ -977,10 +987,11 @@ typedef struct {
   LiVESWidget *end_image;
   LiVESWidget *playarea;
   LiVESWidget *hseparator;
-  LiVESWidget *scrolledwindow;
   LiVESWidget *message_box;
+  LiVESWidget *msg_area;
+  LiVESWidget *msg_scrollbar;
+  LiVESAdjustment *msg_adj;
 
-  LiVESWidget *textview1;
   LiVESWidget *clipsmenu;
   LiVESWidget *eventbox;
   LiVESWidget *eventbox2;
@@ -1063,12 +1074,22 @@ typedef struct {
   LiVESWidget *framecounter;
   LiVESWidget *spinbutton_pb_fps;
   LiVESWidget *vps_label;
-  LiVESWidget *curf_label;
   LiVESWidget *banner;
 
-  // rendered effects
+  // (sub)menus
+  LiVESWidget *files_menu;
+  LiVESWidget *edit_menu;
+  LiVESWidget *play_menu;
   LiVESWidget *effects_menu;
   LiVESWidget *tools_menu;
+  LiVESWidget *audio_menu;
+  LiVESWidget *info_menu;
+  LiVESWidget *advanced_menu;
+  LiVESWidget *vj_menu;
+  LiVESWidget *toys_menu;
+  LiVESWidget *help_menu;
+
+  // rendered effects
   LiVESWidget *utilities_menu;
   LiVESWidget *utilities_submenu;
   LiVESWidget *gens_menu;
@@ -1094,6 +1115,7 @@ typedef struct {
 
   LiVESWidget *resize_menuitem;
 
+  boolean close_keep_frames; ///< special value for when generating to clipboard
   boolean only_close; ///< only close clips - do not exit
   volatile boolean is_exiting; ///< set during shutdown (inverse of only_close then)
 
@@ -1102,6 +1124,7 @@ typedef struct {
 #ifdef ENABLE_JACK
   jack_driver_t *jackd; ///< jack audio playback device
   jack_driver_t *jackd_read; ///< jack audio recorder device
+  boolean jack_inited;
 #define RT_AUDIO
 #else
   void *jackd;  ///< dummy
@@ -1138,9 +1161,9 @@ typedef struct {
 
   mt_opts multi_opts; ///< some multitrack options that survive between mt calls
 
-  int rec_aclip;
-  double rec_avel;
-  double rec_aseek;
+  volatile int rec_aclip;
+  volatile double rec_avel;
+  volatile double rec_aseek;
 
   LiVESMemVTable alt_vtable;
 
@@ -1149,11 +1172,15 @@ typedef struct {
 
   pthread_mutex_t abuf_mutex;  ///< used to synch audio buffer request count - shared between audio and video threads
   pthread_mutex_t abuf_frame_mutex;  ///< used to synch audio buffer for generators
-  pthread_mutex_t data_mutex[FX_KEYS_MAX];  ///< used to prevent data being connected while it is possibly being updated
+  pthread_mutex_t fx_mutex[FX_KEYS_MAX];  ///< used to prevent fx processing when it is scheduled for deinit
   pthread_mutex_t fxd_active_mutex; ///< prevent simultaneous writing to active_dummy by audio and video threads
-  pthread_mutex_t event_list_mutex; /// prevent simultaneous writing to event_list by audio and video threads
-  pthread_mutex_t clip_list_mutex; /// prevent adding/removing to cliplist while another thread could be reading it
-  pthread_mutex_t vpp_stream_mutex; /// prevent from writing audio when stream is closing
+  pthread_mutex_t event_list_mutex; ///< prevent simultaneous writing to event_list by audio and video threads
+  pthread_mutex_t clip_list_mutex; ///< prevent adding/removing to cliplist while another thread could be reading it
+  pthread_mutex_t vpp_stream_mutex; ///< prevent from writing audio when stream is closing
+  pthread_mutex_t cache_buffer_mutex; ///< sync for jack playback termination
+  pthread_mutex_t audio_filewriteend_mutex; ///< sync for ending writing audio to file
+  pthread_mutex_t audio_resync_mutex; ///< sync for audio
+  pthread_mutex_t instance_ref_mutex; ///< refcounting for instances
 
   volatile lives_rfx_t *vrfx_update;
 
@@ -1165,7 +1192,7 @@ typedef struct {
 
   LiVESList *file_buffers;
 
-  float volume; ///< audio volume level (for jack)
+  volatile float volume; ///< audio volume level (for jack and pulse)
 
   int aud_rec_fd; ///< fd of file we are recording audio to
   double rec_end_time;
@@ -1356,8 +1383,10 @@ typedef struct {
   boolean gen_started_play;
   boolean fx_is_auto;
 
-  lives_audio_buf_t *audio_frame_buffer; ///< used for buffering / feeding audio to video generators
+  volatile lives_audio_buf_t *audio_frame_buffer; ///< used for buffering / feeding audio to video generators
+  lives_audio_buf_t *afb[2]; ///< used for buffering / feeding audio to video generators
   int afbuffer_clients;
+  int afbuffer_clients_read;
 
   pthread_t *libthread;
   ulong id;
@@ -1371,11 +1400,30 @@ typedef struct {
 
   uint64_t aud_data_written;
 
-  char *dp_cache;
+  LiVESPixbuf *scrap_pixbuf;
 
+  volatile LiVESWidget *stop_emmission;
+
+  boolean no_context_update;
+
+  weed_plant_t *msg_list;
+  int n_messages;
+  weed_plant_t *ref_message; // weak ref
+  int ref_message_n;
+
+  boolean recovering_files;
   ////////////////////
 } mainwindow;
 
+// weed plants with type >= 512 are reserved for custom use, so let's take advantage of that
+#define WEED_PLANT_LIVES 31337
+
+#define WEED_LEAF_LIVES_SUBTYPE "subtype"
+#define WEED_LEAF_LIVES_MESSAGE_STRING "message_string"
+
+#define LIVES_WEED_SUBTYPE_MESSAGE 1
+#define LIVES_WEED_SUBTYPE_WIDGET 2
+
 extern _palette *palette;
 
 typedef struct {
@@ -1401,6 +1449,7 @@ extern _merge_opts *merge_opts;
 /// 0 for rfx, 1 for rte
 extern LiVESWidget *fx_dialog[2];
 
+#ifndef IS_MINGW
 #define LIVES_SIGKILL SIGKILL
 #define LIVES_SIGINT  SIGINT
 #define LIVES_SIGPIPE SIGPIPE
@@ -1410,6 +1459,17 @@ extern LiVESWidget *fx_dialog[2];
 #define LIVES_SIGHUP  SIGHUP
 #define LIVES_SIGTERM SIGTERM
 #define LIVES_SIGQUIT SIGQUIT
+#else
+#define LIVES_SIGKILL SIGTERM
+#define LIVES_SIGINT  SIGINT
+#define LIVES_SIGPIPE SIGPIPE
+#define LIVES_SIGUSR1 SIGUSR1
+#define LIVES_SIGABRT SIGABRT
+#define LIVES_SIGSEGV SIGSEGV
+#define LIVES_SIGHUP  SIGINT
+#define LIVES_SIGTERM SIGTERM
+#define LIVES_SIGQUIT SIGQUIT
+#endif
 
 #ifdef ENABLE_JACK
 volatile aserver_message_t jack_message;
